@@ -11,13 +11,20 @@ use itertools::Itertools;
 use log::{error, warn};
 use rayon::iter::Either::{Left, Right};
 use rayon::iter::{IntoParallelRefIterator, ParallelIterator};
+use ruff_db::diagnostic::{
+    Annotation, Diagnostic, DiagnosticId, DisplayDiagnosticConfig, Severity, Span,
+};
+use ruff_linter::message::{EmitterContext, create_panic_diagnostic, render_diagnostics};
+use ruff_linter::settings::types::OutputFormat;
+use ruff_markdown::{MarkdownResult, format_code_blocks};
+use ruff_notebook::NotebookIndex;
 use ruff_python_parser::ParseError;
-use rustc_hash::FxHashSet;
+use rustc_hash::{FxHashMap, FxHashSet};
 use thiserror::Error;
 use tracing::debug;
 
 use ruff_db::panic::{PanicError, catch_unwind};
-use ruff_diagnostics::SourceMap;
+use ruff_diagnostics::{Edit, Fix, SourceMap};
 use ruff_linter::fs;
 use ruff_linter::logging::{DisplayParseError, LogLevel};
 use ruff_linter::package::PackageRoot;
@@ -27,14 +34,15 @@ use ruff_linter::source_kind::{SourceError, SourceKind};
 use ruff_linter::warn_user_once;
 use ruff_python_ast::{PySourceType, SourceType};
 use ruff_python_formatter::{FormatModuleError, QuoteStyle, format_module_source, format_range};
-use ruff_source_file::LineIndex;
+use ruff_source_file::{LineIndex, LineRanges, OneIndexed, SourceFileBuilder};
 use ruff_text_size::{TextLen, TextRange, TextSize};
 use ruff_workspace::FormatterSettings;
-use ruff_workspace::resolver::{ResolvedFile, Resolver, match_exclusion, python_files_in_path};
+use ruff_workspace::resolver::{
+    PyprojectConfig, ResolvedFile, Resolver, match_exclusion, python_files_in_path,
+};
 
 use crate::args::{ConfigArguments, FormatArguments, FormatRange};
 use crate::cache::{Cache, FileCacheKey, PackageCacheMap, PackageCaches};
-use crate::resolve::resolve;
 use crate::{ExitStatus, resolve_default_files};
 
 #[derive(Debug, Copy, Clone, is_macro::Is)]
@@ -63,11 +71,14 @@ impl FormatMode {
 pub(crate) fn format(
     cli: FormatArguments,
     config_arguments: &ConfigArguments,
+    pyproject_config: &PyprojectConfig,
 ) -> Result<ExitStatus> {
-    let pyproject_config = resolve(config_arguments, cli.stdin_filename.as_deref())?;
     let mode = FormatMode::from_cli(&cli);
     let files = resolve_default_files(cli.files, false);
-    let (paths, resolver) = python_files_in_path(&files, &pyproject_config, config_arguments)?;
+    let (paths, resolver) = python_files_in_path(&files, pyproject_config, config_arguments)?;
+
+    let output_format = pyproject_config.settings.output_format;
+    let preview = pyproject_config.settings.formatter.preview;
 
     if paths.is_empty() {
         warn_user_once!("No Python files found under the given path(s)");
@@ -112,16 +123,11 @@ pub(crate) fn format(
                     let path = resolved_file.path();
                     let settings = resolver.resolve(path);
 
-                    let source_type = match settings.formatter.extension.get(path) {
-                        None => match SourceType::from(path) {
-                            SourceType::Python(source_type) => source_type,
-                            SourceType::Toml(_) => {
-                                // Ignore any non-Python files.
-                                return None;
-                            }
-                        },
-                        Some(language) => PySourceType::from(language),
-                    };
+                    let source_type = settings.formatter.extension.get_source_type(path);
+                    if source_type.is_toml() {
+                        // Ignore TOML files.
+                        return None;
+                    }
 
                     // Ignore files that are excluded from formatting
                     if (settings.file_resolver.force_exclude || !resolved_file.is_root())
@@ -184,20 +190,31 @@ pub(crate) fn format(
     caches.persist()?;
 
     // Report on any errors.
-    errors.sort_unstable_by(|a, b| a.path().cmp(&b.path()));
+    //
+    // We only convert errors to `Diagnostic`s in `Check` mode with preview enabled, otherwise we
+    // fall back on printing simple messages.
+    if !(preview.is_enabled() && mode.is_check()) {
+        errors.sort_unstable_by(|a, b| a.path().cmp(&b.path()));
 
-    for error in &errors {
-        error!("{error}");
+        for error in &errors {
+            error!("{error}");
+        }
     }
 
     let results = FormatResults::new(results.as_slice(), mode);
-    match mode {
-        FormatMode::Write => {}
-        FormatMode::Check => {
-            results.write_changed(&mut stdout().lock())?;
-        }
-        FormatMode::Diff => {
-            results.write_diff(&mut stdout().lock())?;
+    if config_arguments.log_level > LogLevel::Silent {
+        match mode {
+            FormatMode::Write => {}
+            FormatMode::Check => {
+                if preview.is_enabled() {
+                    results.write_changed_preview(&mut stdout().lock(), output_format, &errors)?;
+                } else {
+                    results.write_changed(&mut stdout().lock())?;
+                }
+            }
+            FormatMode::Diff => {
+                results.write_diff(&mut stdout().lock())?;
+            }
         }
     }
 
@@ -206,7 +223,7 @@ pub(crate) fn format(
         if mode.is_diff() {
             // Allow piping the diff to e.g. a file by writing the summary to stderr
             results.write_summary(&mut stderr().lock())?;
-        } else {
+        } else if !preview.is_enabled() || output_format.is_human_readable() {
             results.write_summary(&mut stdout().lock())?;
         }
     }
@@ -242,7 +259,7 @@ pub(crate) fn format(
 pub(crate) fn format_path(
     path: &Path,
     settings: &FormatterSettings,
-    source_type: PySourceType,
+    source_type: SourceType,
     mode: FormatMode,
     range: Option<FormatRange>,
     cache: Option<&Cache>,
@@ -273,8 +290,7 @@ pub(crate) fn format_path(
     let cache = cache.filter(|_| range.is_none());
 
     // Format the source.
-    let format_result = match format_source(&unformatted, source_type, Some(path), settings, range)?
-    {
+    let format_result = match format_source(&unformatted, Some(path), settings, range)? {
         FormattedSource::Formatted(formatted) => match mode {
             FormatMode::Write => {
                 let mut writer = File::create(path).map_err(|err| {
@@ -295,8 +311,7 @@ pub(crate) fn format_path(
 
                 FormatResult::Formatted
             }
-            FormatMode::Check => FormatResult::Formatted,
-            FormatMode::Diff => FormatResult::Diff {
+            FormatMode::Check | FormatMode::Diff => FormatResult::Diff {
                 unformatted,
                 formatted,
             },
@@ -329,7 +344,7 @@ pub(crate) enum FormattedSource {
 impl From<FormattedSource> for FormatResult {
     fn from(value: FormattedSource) -> Self {
         match value {
-            FormattedSource::Formatted(_) => FormatResult::Formatted,
+            FormattedSource::Formatted { .. } => FormatResult::Formatted,
             FormattedSource::Unchanged => FormatResult::Unchanged,
         }
     }
@@ -339,20 +354,23 @@ impl From<FormattedSource> for FormatResult {
 /// unchanged.
 pub(crate) fn format_source(
     source_kind: &SourceKind,
-    source_type: PySourceType,
     path: Option<&Path>,
     settings: &FormatterSettings,
     range: Option<FormatRange>,
 ) -> Result<FormattedSource, FormatCommandError> {
     match &source_kind {
-        SourceKind::Python(unformatted) => {
-            let options = settings.to_format_options(source_type, unformatted, path);
+        SourceKind::Python {
+            code: unformatted,
+            is_stub,
+        } => {
+            let py_source_type = source_kind.py_source_type();
+            let options = settings.to_format_options(py_source_type, unformatted, path);
 
             let formatted = if let Some(range) = range {
                 let line_index = LineIndex::from_source_text(unformatted);
                 let byte_range = range.to_text_range(unformatted, &line_index);
                 format_range(unformatted, byte_range, options).map(|formatted_range| {
-                    let mut formatted = unformatted.to_string();
+                    let mut formatted = unformatted.clone();
                     formatted.replace_range(
                         std::ops::Range::<usize>::from(formatted_range.source_range()),
                         formatted_range.as_code(),
@@ -382,7 +400,10 @@ pub(crate) fn format_source(
             if formatted.len() == unformatted.len() && formatted == *unformatted {
                 Ok(FormattedSource::Unchanged)
             } else {
-                Ok(FormattedSource::Formatted(SourceKind::Python(formatted)))
+                Ok(FormattedSource::Formatted(SourceKind::Python {
+                    code: formatted,
+                    is_stub: *is_stub,
+                }))
             }
         }
         SourceKind::IpyNotebook(notebook) => {
@@ -391,12 +412,13 @@ pub(crate) fn format_source(
             }
 
             if range.is_some() {
-                return Err(FormatCommandError::RangeFormatNotebook(
+                return Err(FormatCommandError::RangeFormatNotSupported(
                     path.map(Path::to_path_buf),
                 ));
             }
 
-            let options = settings.to_format_options(source_type, notebook.source_code(), path);
+            let options =
+                settings.to_format_options(PySourceType::Ipynb, notebook.source_code(), path);
 
             let mut output: Option<String> = None;
             let mut last: Option<TextSize> = None;
@@ -471,16 +493,36 @@ pub(crate) fn format_source(
                 formatted,
             )))
         }
+        SourceKind::Markdown(unformatted_document) => {
+            if !settings.preview.is_enabled() {
+                return Err(FormatCommandError::MarkdownExperimental(
+                    path.map(Path::to_path_buf),
+                ));
+            }
+
+            if range.is_some() {
+                return Err(FormatCommandError::RangeFormatNotSupported(
+                    path.map(Path::to_path_buf),
+                ));
+            }
+
+            match format_code_blocks(unformatted_document, path, settings) {
+                MarkdownResult::Formatted(formatted) => {
+                    Ok(FormattedSource::Formatted(SourceKind::Markdown(formatted)))
+                }
+                MarkdownResult::Unchanged => Ok(FormattedSource::Unchanged),
+            }
+        }
     }
 }
 
 /// The result of an individual formatting operation.
 #[derive(Debug, Clone, is_macro::Is)]
 pub(crate) enum FormatResult {
-    /// The file was formatted.
+    /// The file was formatted and written back to disk.
     Formatted,
 
-    /// The file was formatted, [`SourceKind`] contains the formatted code
+    /// The file needs to be formatted, as the `formatted` and `unformatted` contents differ.
     Diff {
         unformatted: SourceKind,
         formatted: SourceKind,
@@ -552,7 +594,7 @@ impl<'a> FormatResults<'a> {
             .results
             .iter()
             .filter_map(|result| {
-                if result.result.is_formatted() {
+                if result.result.is_diff() {
                     Some(result.path.as_path())
                 } else {
                     None
@@ -566,6 +608,30 @@ impl<'a> FormatResults<'a> {
         Ok(())
     }
 
+    /// Write a list of the files that would be changed and any errors to the given writer.
+    fn write_changed_preview(
+        &self,
+        f: &mut impl Write,
+        output_format: OutputFormat,
+        errors: &[FormatCommandError],
+    ) -> io::Result<()> {
+        let mut notebook_index = FxHashMap::default();
+        let diagnostics: Vec<_> = errors
+            .iter()
+            .map(Diagnostic::from)
+            .chain(self.to_diagnostics(&mut notebook_index))
+            .sorted_unstable_by(Diagnostic::ruff_start_ordering)
+            .collect();
+
+        let context = EmitterContext::new(&notebook_index);
+        let config = DisplayDiagnosticConfig::new("ruff")
+            .hide_severity(true)
+            .show_fix_diff(true)
+            .color(!cfg!(test) && colored::control::SHOULD_COLORIZE.should_colorize());
+
+        render_diagnostics(f, output_format, config, &context, &diagnostics)
+    }
+
     /// Write a summary of the formatting results to the given writer.
     fn write_summary(&self, f: &mut impl Write) -> io::Result<()> {
         // Compute the number of changed and unchanged files.
@@ -628,6 +694,155 @@ impl<'a> FormatResults<'a> {
             Ok(())
         }
     }
+
+    /// Convert formatted files into [`Diagnostic`]s.
+    fn to_diagnostics(
+        &self,
+        notebook_index: &mut FxHashMap<String, NotebookIndex>,
+    ) -> impl Iterator<Item = Diagnostic> {
+        /// The number of unmodified context lines rendered in diffs.
+        ///
+        /// Note that this should be kept in sync with the argument to `TextDiff::grouped_ops` in
+        /// the diff rendering in `ruff_db` (currently 3). The `similar` crate uses two times that
+        /// argument as a cutoff for rendering unmodified lines.
+        const CONTEXT_LINES: u32 = 6;
+
+        self.results.iter().filter_map(|result| {
+            let (unformatted, formatted) = match &result.result {
+                FormatResult::Skipped | FormatResult::Unchanged => return None,
+                FormatResult::Diff {
+                    unformatted,
+                    formatted,
+                } => (unformatted, formatted),
+                FormatResult::Formatted => {
+                    debug_assert!(
+                        false,
+                        "Expected `FormatResult::Diff` for changed files in check mode"
+                    );
+                    return None;
+                }
+            };
+
+            let mut diagnostic = Diagnostic::new(
+                DiagnosticId::Unformatted,
+                Severity::Error,
+                "File would be reformatted",
+            );
+
+            // Locate the first and last characters that differ to use as the diagnostic
+            // range and to narrow the `Edit` range.
+            let modified_range = ModifiedRange::new(unformatted, formatted);
+
+            let path = result.path.to_string_lossy();
+            // For scripts, this is a single `Edit` using the `ModifiedRange` above, but notebook
+            // edits must be split by cell in order to render them as diffs.
+            //
+            // We also attempt to estimate the line number width for aligning the
+            // annotate-snippets header. This is only an estimate because we don't actually know
+            // if the maximum line number present in the document will be rendered as part of
+            // the diff, either as a changed line or as an unchanged context line. For
+            // notebooks, we refine our estimate by checking the number of lines in each cell
+            // individually, otherwise we could use `formatted.source_code().count_lines(...)`
+            // in both cases.
+            let (fix, line_count) = if let SourceKind::IpyNotebook(formatted) = formatted
+                && let SourceKind::IpyNotebook(unformatted) = unformatted
+            {
+                notebook_index.insert(path.to_string(), unformatted.index().clone());
+
+                let mut edits = formatted
+                    .cell_offsets()
+                    .ranges()
+                    .zip(unformatted.cell_offsets().ranges())
+                    .filter_map(|(formatted_range, unformatted_range)| {
+                        // Filter out cells that weren't modified. We use `intersect` instead of
+                        // `contains_range` because the full modified range might start or end in
+                        // the middle of a cell:
+                        //
+                        // ```
+                        // | cell 1 | cell 2 | cell 3 |
+                        //     |----------------| modified range
+                        // ```
+                        //
+                        // The intersection will be `Some` for all three cells in this case.
+                        if modified_range
+                            .unformatted
+                            .intersect(unformatted_range)
+                            .is_some()
+                        {
+                            let formatted = &formatted.source_code()[formatted_range];
+                            let edit = if formatted.is_empty() {
+                                Edit::range_deletion(unformatted_range)
+                            } else {
+                                Edit::range_replacement(formatted.to_string(), unformatted_range)
+                            };
+                            Some(edit)
+                        } else {
+                            None
+                        }
+                    });
+
+                let fix = Fix::safe_edits(
+                    edits
+                        .next()
+                        .expect("Formatted files must have at least one edit"),
+                    edits,
+                );
+                let source = formatted.source_code();
+                let line_count = formatted
+                    .cell_offsets()
+                    .ranges()
+                    .filter_map(|range| {
+                        if modified_range.formatted.contains_range(range) {
+                            Some(source.count_lines(range))
+                        } else {
+                            None
+                        }
+                    })
+                    .max()
+                    .unwrap_or_default();
+                (fix, line_count)
+            } else {
+                let formatted_code = &formatted.source_code()[modified_range.formatted];
+                let edit = if formatted_code.is_empty() {
+                    Edit::range_deletion(modified_range.unformatted)
+                } else {
+                    Edit::range_replacement(formatted_code.to_string(), modified_range.unformatted)
+                };
+                let fix = Fix::safe_edit(edit);
+                let line_count = formatted
+                    .source_code()
+                    .count_lines(TextRange::up_to(modified_range.formatted.end()));
+                (fix, line_count)
+            };
+
+            let source_file = SourceFileBuilder::new(path, unformatted.source_code()).finish();
+            let span = Span::from(source_file).with_range(modified_range.unformatted);
+            let mut annotation = Annotation::primary(span);
+            annotation.hide_snippet(true);
+            diagnostic.annotate(annotation);
+            diagnostic.set_fix(fix);
+
+            // TODO(brent) this offset is a hack to get the header of the diagnostic message, which
+            // is rendered by our fork of `annotate-snippets`, to align with our manually-rendered
+            // diff. `annotate-snippets` computes the alignment of the arrow in the header based on
+            // the maximum line number width in its rendered snippet. However, we don't have a
+            // reasonable range to underline in an annotation, so we don't send `annotate-snippets`
+            // a snippet to measure. If we commit to staying on our fork, a more robust way of
+            // handling this would be to move the diff rendering in
+            // `ruff_db::diagnostic::render::full` into `annotate-snippets`, likely as another
+            // `DisplayLine` variant and update the `lineno_width` calculation in
+            // `DisplayList::fmt`. That would handle this offset "automatically."
+            let line_count = (line_count + CONTEXT_LINES).min(
+                formatted
+                    .source_code()
+                    .count_lines(TextRange::up_to(formatted.source_code().text_len())),
+            );
+            let lines = OneIndexed::new(line_count as usize).unwrap_or_default();
+            diagnostic.set_header_offset(lines.digits().get());
+
+            Some(diagnostic)
+        })
+    }
 }
 
 /// An error that can occur while formatting a set of files.
@@ -639,8 +854,8 @@ pub(crate) enum FormatCommandError {
     Read(Option<PathBuf>, SourceError),
     Format(Option<PathBuf>, FormatModuleError),
     Write(Option<PathBuf>, SourceError),
-    Diff(Option<PathBuf>, io::Error),
-    RangeFormatNotebook(Option<PathBuf>),
+    RangeFormatNotSupported(Option<PathBuf>),
+    MarkdownExperimental(Option<PathBuf>),
 }
 
 impl FormatCommandError {
@@ -658,12 +873,59 @@ impl FormatCommandError {
             | Self::Read(path, _)
             | Self::Format(path, _)
             | Self::Write(path, _)
-            | Self::Diff(path, _)
-            | Self::RangeFormatNotebook(path) => path.as_deref(),
+            | Self::RangeFormatNotSupported(path)
+            | Self::MarkdownExperimental(path) => path.as_deref(),
         }
     }
 }
 
+impl From<&FormatCommandError> for Diagnostic {
+    fn from(error: &FormatCommandError) -> Self {
+        let annotation = error.path().map(|path| {
+            let file = SourceFileBuilder::new(path.to_string_lossy(), "").finish();
+            let span = Span::from(file);
+            let mut annotation = Annotation::primary(span);
+            annotation.hide_snippet(true);
+            annotation
+        });
+
+        let mut diagnostic = match error {
+            FormatCommandError::Ignore(error) => {
+                Diagnostic::new(DiagnosticId::Io, Severity::Error, error)
+            }
+            FormatCommandError::Parse(display_parse_error) => Diagnostic::new(
+                DiagnosticId::InvalidSyntax,
+                Severity::Error,
+                &display_parse_error.error().error,
+            ),
+            FormatCommandError::Panic(path, panic_error) => {
+                return create_panic_diagnostic(panic_error, path.as_deref());
+            }
+            FormatCommandError::Read(_, source_error)
+            | FormatCommandError::Write(_, source_error) => {
+                Diagnostic::new(DiagnosticId::Io, Severity::Error, source_error)
+            }
+            FormatCommandError::Format(_, format_module_error) => format_module_error.into(),
+            FormatCommandError::RangeFormatNotSupported(_) => Diagnostic::new(
+                DiagnosticId::InvalidCliOption,
+                Severity::Error,
+                "Range formatting is only supported for Python files.",
+            ),
+            FormatCommandError::MarkdownExperimental(_) => Diagnostic::new(
+                DiagnosticId::PreviewFeature,
+                Severity::Warning,
+                "Markdown formatting is experimental, enable preview mode.",
+            ),
+        };
+
+        if let Some(annotation) = annotation {
+            diagnostic.annotate(annotation);
+        }
+
+        diagnostic
+    }
+}
+
 impl Display for FormatCommandError {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         match self {
@@ -731,28 +993,11 @@ impl Display for FormatCommandError {
                     write!(f, "{header} {err}", header = "Failed to format:".bold())
                 }
             }
-            Self::Diff(path, err) => {
+            Self::RangeFormatNotSupported(path) => {
                 if let Some(path) = path {
                     write!(
                         f,
-                        "{}{}{} {err}",
-                        "Failed to generate diff for ".bold(),
-                        fs::relativize_path(path).bold(),
-                        ":".bold()
-                    )
-                } else {
-                    write!(
-                        f,
-                        "{header} {err}",
-                        header = "Failed to generate diff:".bold(),
-                    )
-                }
-            }
-            Self::RangeFormatNotebook(path) => {
-                if let Some(path) = path {
-                    write!(
-                        f,
-                        "{header}{path}{colon} Range formatting isn't supported for notebooks.",
+                        "{header}{path}{colon} Range formatting is only supported for Python files.",
                         header = "Failed to format ".bold(),
                         path = fs::relativize_path(path).bold(),
                         colon = ":".bold()
@@ -760,7 +1005,24 @@ impl Display for FormatCommandError {
                 } else {
                     write!(
                         f,
-                        "{header} Range formatting isn't supported for notebooks",
+                        "{header} Range formatting is only supported for Python files",
+                        header = "Failed to format:".bold()
+                    )
+                }
+            }
+            Self::MarkdownExperimental(path) => {
+                if let Some(path) = path {
+                    write!(
+                        f,
+                        "{header}{path}{colon} Markdown formatting is experimental, enable preview mode.",
+                        header = "Failed to format ".bold(),
+                        path = fs::relativize_path(path).bold(),
+                        colon = ":".bold()
+                    )
+                } else {
+                    write!(
+                        f,
+                        "{header} Markdown formatting is experimental, enable preview mode",
                         header = "Failed to format:".bold()
                     )
                 }
@@ -792,6 +1054,54 @@ impl Display for FormatCommandError {
     }
 }
 
+#[derive(Debug)]
+struct ModifiedRange {
+    unformatted: TextRange,
+    formatted: TextRange,
+}
+
+impl ModifiedRange {
+    /// Determine the range that differs between `unformatted` and `formatted`.
+    ///
+    /// If the two inputs are equal, the returned ranges will be empty.
+    fn new(unformatted: &SourceKind, formatted: &SourceKind) -> Self {
+        let unformatted = unformatted.source_code();
+        let formatted = formatted.source_code();
+
+        let mut prefix_length = TextSize::ZERO;
+        for (unformatted, formatted) in unformatted.chars().zip(formatted.chars()) {
+            if unformatted != formatted {
+                break;
+            }
+            prefix_length += unformatted.text_len();
+        }
+
+        // For the ends of the ranges, track the length of the common suffix and then subtract that
+        // from each total text length. Unlike for `start`, the character offsets are very unlikely
+        // to be equal, so they need to be treated separately.
+        let mut suffix_length = TextSize::ZERO;
+        for (old, new) in unformatted[prefix_length.to_usize()..]
+            .chars()
+            .rev()
+            .zip(formatted[prefix_length.to_usize()..].chars().rev())
+        {
+            if old != new {
+                break;
+            }
+            suffix_length += old.text_len();
+        }
+
+        let unformatted_range =
+            TextRange::new(prefix_length, unformatted.text_len() - suffix_length);
+        let formatted_range = TextRange::new(prefix_length, formatted.text_len() - suffix_length);
+
+        Self {
+            unformatted: unformatted_range,
+            formatted: formatted_range,
+        }
+    }
+}
+
 pub(super) fn warn_incompatible_formatter_settings(resolver: &Resolver) {
     // First, collect all rules that are incompatible regardless of the linter-specific settings.
     let mut incompatible_rules = FxHashSet::default();
@@ -963,3 +1273,153 @@ pub(super) fn warn_incompatible_formatter_settings(resolver: &Resolver) {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use std::io;
+    use std::ops::Range;
+    use std::path::PathBuf;
+
+    use ignore::Error;
+    use insta::assert_snapshot;
+
+    use ruff_db::panic::catch_unwind;
+    use ruff_linter::logging::DisplayParseError;
+    use ruff_linter::source_kind::{SourceError, SourceKind};
+    use ruff_python_formatter::FormatModuleError;
+    use ruff_python_parser::{ParseError, ParseErrorType};
+    use ruff_text_size::{TextRange, TextSize};
+    use test_case::test_case;
+
+    use crate::commands::format::{FormatCommandError, FormatMode, FormatResults, ModifiedRange};
+
+    #[test]
+    fn error_diagnostics() -> anyhow::Result<()> {
+        let path = PathBuf::from("test.py");
+        let source_kind = SourceKind::Python {
+            code: "1".to_string(),
+            is_stub: false,
+        };
+
+        let panic_error = catch_unwind(|| {
+            panic!("Test panic for FormatCommandError");
+        })
+        .unwrap_err();
+
+        let errors = [
+            FormatCommandError::Ignore(Error::WithPath {
+                path: path.clone(),
+                err: Box::new(Error::Io(io::Error::new(
+                    io::ErrorKind::PermissionDenied,
+                    "Permission denied",
+                ))),
+            }),
+            FormatCommandError::Parse(DisplayParseError::from_source_kind(
+                ParseError {
+                    error: ParseErrorType::UnexpectedIndentation,
+                    location: TextRange::default(),
+                },
+                Some(path.clone()),
+                &source_kind,
+            )),
+            FormatCommandError::Panic(Some(path.clone()), Box::new(panic_error)),
+            FormatCommandError::Read(
+                Some(path.clone()),
+                SourceError::Io(io::Error::new(io::ErrorKind::NotFound, "File not found")),
+            ),
+            FormatCommandError::Format(
+                Some(path.clone()),
+                FormatModuleError::ParseError(ParseError {
+                    error: ParseErrorType::EmptySlice,
+                    location: TextRange::default(),
+                }),
+            ),
+            FormatCommandError::Write(
+                Some(path.clone()),
+                SourceError::Io(io::Error::new(
+                    io::ErrorKind::PermissionDenied,
+                    "Cannot write to file",
+                )),
+            ),
+            FormatCommandError::RangeFormatNotSupported(Some(path)),
+        ];
+
+        let results = FormatResults::new(&[], FormatMode::Check);
+        let mut buf = Vec::new();
+        results.write_changed_preview(
+            &mut buf,
+            ruff_linter::settings::types::OutputFormat::Full,
+            &errors,
+        )?;
+
+        let mut settings = insta::Settings::clone_current();
+        settings.add_filter(r"(Panicked at) [^:]+:\d+:\d+", "$1 <location>");
+        let _s = settings.bind_to_scope();
+
+        assert_snapshot!(str::from_utf8(&buf)?, @r"
+        io: test.py: Permission denied
+        --> test.py:1:1
+
+        invalid-syntax: Unexpected indentation
+        --> test.py:1:1
+
+        io: File not found
+        --> test.py:1:1
+
+        internal-error: Expected index or slice expression
+        --> test.py:1:1
+
+        io: Cannot write to file
+        --> test.py:1:1
+
+        invalid-cli-option: Range formatting is only supported for Python files.
+        --> test.py:1:1
+
+        panic: Panicked at <location> when checking `test.py`: `Test panic for FormatCommandError`
+        --> test.py:1:1
+        info: This indicates a bug in Ruff.
+        info: If you could open an issue at https://github.com/astral-sh/ruff/issues/new?title=%5Bpanic%5D, we'd be very appreciative!
+        info: run with `RUST_BACKTRACE=1` environment variable to show the full backtrace information
+        ");
+
+        Ok(())
+    }
+
+    #[test_case("abcdef", "abcXYdef", 3..3, 3..5; "insertion")]
+    #[test_case("abcXYdef", "abcdef", 3..5, 3..3; "deletion")]
+    #[test_case("abcXdef", "abcYdef", 3..4, 3..4; "modification")]
+    #[test_case("abc", "abcX", 3..3, 3..4; "strict_prefix")]
+    #[test_case("", "", 0..0, 0..0; "empty")]
+    #[test_case("abc", "abc", 3..3, 3..3; "equal")]
+    fn modified_range(
+        unformatted: &str,
+        formatted: &str,
+        expect_unformatted: Range<u32>,
+        expect_formatted: Range<u32>,
+    ) {
+        let mr = ModifiedRange::new(
+            &SourceKind::Python {
+                code: unformatted.to_string(),
+                is_stub: false,
+            },
+            &SourceKind::Python {
+                code: formatted.to_string(),
+                is_stub: false,
+            },
+        );
+        assert_eq!(
+            mr.unformatted,
+            TextRange::new(
+                TextSize::new(expect_unformatted.start),
+                TextSize::new(expect_unformatted.end)
+            )
+        );
+        assert_eq!(
+            mr.formatted,
+            TextRange::new(
+                TextSize::new(expect_formatted.start),
+                TextSize::new(expect_formatted.end)
+            )
+        );
+    }
+}
