@@ -83,13 +83,17 @@ fn benchmark_linter(mut group: BenchmarkGroup, settings: &LinterSettings) {
                         assert!(parsed.has_valid_syntax());
 
                         let path = case.path();
+                        let py_source_type = PySourceType::from(path.as_path());
                         lint_only(
                             &path,
                             None,
                             settings,
                             flags::Noqa::Enabled,
-                            &SourceKind::Python(case.code().to_string()),
-                            PySourceType::from(path.as_path()),
+                            &SourceKind::Python {
+                                code: case.code().to_string(),
+                                is_stub: py_source_type.is_stub(),
+                            },
+                            py_source_type,
                             ParseSource::Precomputed(parsed),
                         )
                     },
