@@ -39,6 +39,7 @@ use crate::checkers::ast::Checker;
 /// ## References
 /// - [Python documentation: The `__hash__` method](https://docs.python.org/3/reference/datamodel.html#object.__hash__)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "0.6.0")]
 pub(crate) struct InvalidHashReturnType;
 
 impl Violation for InvalidHashReturnType {
@@ -63,10 +64,10 @@ pub(crate) fn invalid_hash_return(checker: &Checker, function_def: &ast::StmtFun
     }
 
     // Determine the terminal behavior (i.e., implicit return, no return, etc.).
-    let terminal = Terminal::from_function(function_def);
+    let terminal = Terminal::from_function(function_def, checker.semantic());
 
     // If every control flow path raises an exception, ignore the function.
-    if terminal == Terminal::Raise {
+    if terminal.is_always_raise() {
         return;
     }
 
