@@ -1,22 +1,23 @@
 use std::borrow::Cow;
 
+use ruff_python_ast::SourceType;
 use rustc_hash::FxHashMap;
 
+use crate::{
+    PositionEncoding,
+    edit::{Replacement, ToRangeExt},
+    resolve::is_document_excluded_for_linting,
+    session::DocumentQuery,
+};
+use ruff_linter::package::PackageRoot;
 use ruff_linter::{
-    linter::{FixerResult, LinterResult},
+    linter::FixerResult,
     packaging::detect_package_root,
-    settings::{flags, types::UnsafeFixes, LinterSettings},
+    settings::{LinterSettings, flags},
 };
 use ruff_notebook::SourceValue;
 use ruff_source_file::LineIndex;
 
-use crate::{
-    edit::{Replacement, ToRangeExt},
-    resolve::is_document_excluded,
-    session::DocumentQuery,
-    PositionEncoding,
-};
-
 /// A simultaneous fix made across a single text document or among an arbitrary
 /// number of notebook cells.
 pub(crate) type Fixes = FxHashMap<lsp_types::Url, Vec<lsp_types::TextEdit>>;
@@ -27,32 +28,35 @@ pub(crate) fn fix_all(
     encoding: PositionEncoding,
 ) -> crate::Result<Fixes> {
     let source_kind = query.make_source_kind();
-
-    let file_resolver_settings = query.settings().file_resolver();
-    let document_path = query.file_path();
+    let settings = query.settings();
+    let document_path = query.virtual_file_path();
 
     // If the document is excluded, return an empty list of fixes.
-    let package = if let Some(document_path) = document_path.as_ref() {
-        if is_document_excluded(
-            document_path,
-            file_resolver_settings,
-            Some(linter_settings),
-            None,
-        ) {
-            return Ok(Fixes::default());
-        }
+    if is_document_excluded_for_linting(
+        &document_path,
+        &settings.file_resolver,
+        linter_settings,
+        query.text_document_language_id(),
+    ) {
+        return Ok(Fixes::default());
+    }
 
+    let file_path = query.file_path();
+    let package = if let Some(file_path) = &file_path {
         detect_package_root(
-            document_path
+            file_path
                 .parent()
                 .expect("a path to a document should have a parent path"),
             &linter_settings.namespace_packages,
         )
+        .map(PackageRoot::root)
     } else {
         None
     };
 
-    let source_type = query.source_type();
+    let SourceType::Python(source_type) = query.source_type() else {
+        return Ok(Fixes::default());
+    };
 
     // We need to iteratively apply all safe fixes onto a single file and then
     // create a diff between the modified file and the original source to use as a single workspace
@@ -62,21 +66,19 @@ pub(crate) fn fix_all(
     // which is inconsistent with how `ruff check --fix` works.
     let FixerResult {
         transformed,
-        result: LinterResult {
-            has_syntax_error, ..
-        },
+        result,
         ..
     } = ruff_linter::linter::lint_fix(
-        &query.virtual_file_path(),
+        &document_path,
         package,
         flags::Noqa::Enabled,
-        UnsafeFixes::Disabled,
+        settings.unsafe_fixes,
         linter_settings,
         &source_kind,
         source_type,
     )?;
 
-    if has_syntax_error {
+    if result.has_invalid_syntax() {
         // If there's a syntax error, then there won't be any fixes to apply.
         return Ok(Fixes::default());
     }
@@ -123,11 +125,7 @@ pub(crate) fn fix_all(
             fixes.insert(
                 url.clone(),
                 vec![lsp_types::TextEdit {
-                    range: source_range.to_range(
-                        source_kind.source_code(),
-                        &source_index,
-                        encoding,
-                    ),
+                    range: source_range.to_range(&source, &source_index, encoding),
                     new_text: modified[modified_range].to_owned(),
                 }],
             );
