@@ -127,9 +127,10 @@ impl Index {
     pub(super) fn key_from_url(&self, url: Url) -> DocumentKey {
         if self.notebook_cells.contains_key(&url) {
             DocumentKey::NotebookCell(url)
-        } else if Path::new(url.path())
-            .extension()
-            .is_some_and(|ext| ext.eq_ignore_ascii_case("ipynb"))
+        } else if self
+            .documents
+            .get(&url)
+            .is_some_and(|controller| controller.as_notebook().is_some())
         {
             DocumentKey::Notebook(url)
         } else {
@@ -187,12 +188,9 @@ impl Index {
             anyhow!("Failed to convert workspace URL to file path: {workspace_url}")
         })?;
 
-        self.settings.remove(&workspace_path).ok_or_else(|| {
-            anyhow!(
-                "Tried to remove non-existent workspace URI {}",
-                workspace_url
-            )
-        })?;
+        self.settings
+            .remove(&workspace_path)
+            .ok_or_else(|| anyhow!("Tried to remove non-existent workspace URI {workspace_url}"))?;
 
         // O(n) complexity, which isn't ideal... but this is an uncommon operation.
         self.documents
@@ -330,7 +328,7 @@ impl Index {
         };
 
         let Some(_) = self.documents.remove(&url) else {
-            anyhow::bail!("tried to close document that didn't exist at {}", url)
+            anyhow::bail!("tried to close document that didn't exist at {url}")
         };
         Ok(())
     }
@@ -351,7 +349,7 @@ impl Index {
             anyhow::bail!("Tried to open unavailable document `{key}`");
         };
         let Some(controller) = self.documents.get_mut(&url) else {
-            anyhow::bail!("Document controller not available at `{}`", url);
+            anyhow::bail!("Document controller not available at `{url}`");
         };
         Ok(controller)
     }
@@ -566,9 +564,10 @@ impl DocumentQuery {
     /// Generate a source kind used by the linter.
     pub(crate) fn make_source_kind(&self) -> ruff_linter::source_kind::SourceKind {
         match self {
-            Self::Text { document, .. } => {
-                ruff_linter::source_kind::SourceKind::Python(document.contents().to_string())
-            }
+            Self::Text { document, .. } => ruff_linter::source_kind::SourceKind::Python {
+                code: document.contents().to_string(),
+                is_stub: ruff_python_ast::PySourceType::from(self.virtual_file_path()).is_stub(),
+            },
             Self::Notebook { notebook, .. } => {
                 ruff_linter::source_kind::SourceKind::ipy_notebook(notebook.make_ruff_notebook())
             }
@@ -584,10 +583,12 @@ impl DocumentQuery {
     }
 
     /// Get the source type of the document associated with this query.
-    pub(crate) fn source_type(&self) -> ruff_python_ast::PySourceType {
+    pub(crate) fn source_type(&self) -> ruff_python_ast::SourceType {
         match self {
-            Self::Text { .. } => ruff_python_ast::PySourceType::from(self.virtual_file_path()),
-            Self::Notebook { .. } => ruff_python_ast::PySourceType::Ipynb,
+            Self::Text { .. } => ruff_python_ast::SourceType::from(self.virtual_file_path()),
+            Self::Notebook { .. } => {
+                ruff_python_ast::SourceType::Python(ruff_python_ast::PySourceType::Ipynb)
+            }
         }
     }
 
@@ -619,7 +620,7 @@ impl DocumentQuery {
     /// Get the path for the document selected by this query, ignoring whether the file exists on disk.
     ///
     /// Returns the URL's path if this is an unsaved (untitled) document.
-    pub(crate) fn virtual_file_path(&self) -> Cow<Path> {
+    pub(crate) fn virtual_file_path(&self) -> Cow<'_, Path> {
         self.file_path()
             .map(Cow::Owned)
             .unwrap_or_else(|| Cow::Borrowed(Path::new(self.file_url().path())))
