@@ -28,7 +28,9 @@ use std::ops::Deref;
 use std::sync::Arc;
 use thiserror::Error;
 use ty_combine::Combine;
-use ty_module_resolver::{SearchPathSettings, SearchPathSettingsError, SearchPaths};
+use ty_module_resolver::{
+    ModuleGlobSet, ModuleGlobSetBuilder, SearchPathSettings, SearchPathSettingsError, SearchPaths,
+};
 use ty_python_semantic::lint::{Level, LintSource, RuleSelection};
 use ty_python_semantic::{
     AnalysisSettings, MisconfigurationMode, ProgramSettings, PythonEnvironment, PythonPlatform,
@@ -63,6 +65,7 @@ pub struct Options {
 
     /// Configures the enabled rules and their severity.
     ///
+    /// The keys are either rule names or `all` to set a default severity for all rules.
     /// See [the rules documentation](https://ty.dev/rules) for a list of all available rules.
     ///
     /// Valid severities are:
@@ -74,7 +77,7 @@ pub struct Options {
     #[serde(skip_serializing_if = "Option::is_none")]
     #[option(
         default = r#"{...}"#,
-        value_type = r#"dict[RuleName, "ignore" | "warn" | "error"]"#,
+        value_type = r#"dict[RuleName | "all", "ignore" | "warn" | "error"]"#,
         example = r#"
             [tool.ty.rules]
             possibly-unresolved-reference = "warn"
@@ -181,14 +184,13 @@ impl Options {
             }
         };
 
-        let self_site_packages = self_environment_search_paths(
+        let self_environment = self_environment_search_paths(
             python_environment
                 .as_ref()
                 .map(ty_python_semantic::PythonEnvironment::origin)
                 .cloned(),
             system,
-        )
-        .unwrap_or_default();
+        );
 
         let site_packages_paths = if let Some(python_environment) = python_environment.as_ref() {
             let site_packages_paths = python_environment
@@ -207,10 +209,19 @@ impl Options {
                     }
                 }
             };
-            self_site_packages.concatenate(site_packages_paths)
+            match self_environment {
+                // When ty is installed in a virtual environment (e.g., `uvx --with ...`),
+                // the self-environment takes priority over the discovered environment.
+                Some((self_site_packages, true)) => {
+                    self_site_packages.concatenate(site_packages_paths)
+                }
+                // When ty is installed in a system Python, do not include the system
+                // Python's site-packages if there's a discovered project environment.
+                Some((_, false)) | None => site_packages_paths,
+            }
         } else {
             tracing::debug!("No virtual environment found");
-            self_site_packages
+            self_environment.map(|(paths, _)| paths).unwrap_or_default()
         };
 
         let real_stdlib_path = python_environment.as_ref().and_then(|python_environment| {
@@ -279,47 +290,45 @@ impl Options {
                 .collect()
         } else {
             let mut roots = vec![];
-            let src = project_root.join("src");
+            let is_package = |dir: &SystemPath| {
+                system.is_file(&dir.join("__init__.py"))
+                    || system.is_file(&dir.join("__init__.pyi"))
+            };
 
-            if system.is_directory(&src) {
-                // Default to `src` and the project root if `src` exists and the root hasn't been specified.
-                // This corresponds to the `src-layout`
+            // Check for `./src` directory (src-layout)
+            let src = project_root.join("src");
+            if system.is_directory(&src) && !is_package(&src) {
                 tracing::debug!(
-                    "Including `.` and `./src` in `environment.root` because a `./src` directory exists"
+                    "Including `./src` in `environment.root` because a `./src` directory exists and is not a package"
                 );
                 roots.push(src);
-            } else if system.is_directory(&project_root.join(project_name).join(project_name)) {
-                // `src-layout` but when the folder isn't called `src` but has the same name as the project.
-                // For example, the "src" folder for `psycopg` is called `psycopg` and the python files are in `psycopg/psycopg/_adapters_map.py`
-                tracing::debug!(
-                    "Including `.` and `/{project_name}` in `environment.root` because a `./{project_name}/{project_name}` directory exists"
-                );
-
-                roots.push(project_root.join(project_name));
-            } else {
-                // Default to a [flat project structure](https://packaging.python.org/en/latest/discussions/src-layout-vs-flat-layout/).
-                tracing::debug!("Including `.` in `environment.root`");
             }
 
-            let python = project_root.join("python");
-            if system.is_directory(&python)
-                && !system.is_file(&python.join("__init__.py"))
-                && !system.is_file(&python.join("__init__.pyi"))
-                && !roots.contains(&python)
+            // Check for `./<project-name>/<project-name>` directory (src-layout with project-named folder)
+            // For example, the "src" folder for `psycopg` is called `psycopg` and the python files are in `psycopg/psycopg/_adapters_map.py`
+            let project_name_dir = project_root.join(project_name);
+            if system.is_directory(&project_name_dir.join(project_name))
+                && !is_package(&project_name_dir)
+                && !roots.contains(&project_name_dir)
             {
-                // If a `./python` directory exists, include it as a source root. This is the recommended layout
-                // for maturin-based rust/python projects [1].
-                //
-                // https://github.com/PyO3/maturin/blob/979fe1db42bb9e58bc150fa6fc45360b377288bf/README.md?plain=1#L88-L99
                 tracing::debug!(
-                    "Including `./python` in `environment.root` because a `./python` directory exists"
+                    "Including `./{project_name}` in `environment.root` because a `./{project_name}/{project_name}` directory exists and `./{project_name}` is not a package"
                 );
+                roots.push(project_name_dir);
+            }
 
+            // Check for `./python` directory (maturin-based rust/python projects)
+            // https://github.com/PyO3/maturin/blob/979fe1db42bb9e58bc150fa6fc45360b377288bf/README.md?plain=1#L88-L99
+            let python = project_root.join("python");
+            if system.is_directory(&python) && !is_package(&python) && !roots.contains(&python) {
+                tracing::debug!(
+                    "Including `./python` in `environment.root` because a `./python` directory exists and is not a package"
+                );
                 roots.push(python);
             }
 
-            // The project root should always be included, and should always come
-            // after any subdirectories such as `./src`, `./tests` and/or `./python`.
+            // The project root is always included, and should always come last
+            // (after any subdirectories such as `./src`, `./<project-name>`, and/or `./python`).
             roots.push(project_root.to_path_buf());
 
             roots
@@ -439,7 +448,19 @@ impl Options {
                 color: colored::control::SHOULD_COLORIZE.should_colorize(),
             })?;
 
-        let analysis = self.analysis.or_default().to_settings();
+        let mut analysis_diagnostics = Vec::new();
+        let analysis = self
+            .analysis
+            .or_default()
+            .to_settings(db, &mut analysis_diagnostics);
+
+        if let Some(diagnostic) = analysis_diagnostics.into_iter().next() {
+            return Err(ToSettingsError {
+                diagnostic: Box::new(diagnostic),
+                output_format: terminal.output_format,
+                color: colored::control::SHOULD_COLORIZE.should_colorize(),
+            });
+        }
 
         let overrides = self
             .to_overrides_settings(db, project_root, &mut diagnostics)
@@ -480,8 +501,13 @@ impl Options {
         let mut overrides = Vec::with_capacity(override_options.len());
 
         for override_option in override_options {
-            let override_instance =
-                override_option.to_override(db, project_root, self.rules.as_ref(), diagnostics)?;
+            let override_instance = override_option.to_override(
+                db,
+                project_root,
+                self.rules.as_ref(),
+                self.analysis.as_ref(),
+                diagnostics,
+            )?;
 
             if let Some(value) = override_instance {
                 overrides.push(value);
@@ -500,10 +526,15 @@ impl Options {
 ///
 /// Since ty may be executed from an arbitrary non-Python location, errors during discovery of ty's
 /// environment are not raised, instead [`None`] is returned.
+///
+/// Returns a tuple of (`site_packages`, `is_virtual_env`). When the self-environment is a virtual
+/// environment (e.g., `uvx --with ...`), it takes priority over other environments.
+/// When it's a system Python and there's a project environment (like `.venv`), the system
+/// Python's site-packages are excluded entirely.
 fn self_environment_search_paths(
     existing_origin: Option<SysPrefixPathOrigin>,
     system: &dyn System,
-) -> Option<SitePackagesPaths> {
+) -> Option<(SitePackagesPaths, bool)> {
     if existing_origin.is_some_and(|origin| !origin.allows_concatenation_with_self_environment()) {
         return None;
     }
@@ -517,15 +548,17 @@ fn self_environment_search_paths(
         .inspect_err(|err| tracing::debug!("Failed to discover ty's environment: {err}"))
         .ok()?;
 
+    let is_virtual_env = environment.is_virtual();
+
     let search_paths = environment
         .site_packages_paths(system)
         .inspect_err(|err| {
             tracing::debug!("Failed to discover site-packages in ty's environment: {err}");
         })
-        .ok();
+        .ok()?;
 
     tracing::debug!("Using site-packages from ty's environment");
-    search_paths
+    Some((search_paths, is_virtual_env))
 }
 
 #[derive(
@@ -547,14 +580,13 @@ pub struct EnvironmentOptions {
     ///
     /// Accepts a list of directory paths searched in priority order (first has highest priority).
     ///
-    /// If left unspecified, ty will try to detect common project layouts and initialize `root` accordingly:
+    /// If left unspecified, ty will try to detect common project layouts and initialize `root` accordingly.
+    /// The project root (`.`) is always included. Additionally, the following directories are included
+    /// if they exist and are not packages (i.e. they do not contain `__init__.py` or `__init__.pyi` files):
     ///
-    /// * if a `./src` directory exists, include `.` and `./src` in the first party search path (src layout or flat)
-    /// * if a `./<project-name>/<project-name>` directory exists, include `.` and `./<project-name>` in the first party search path
-    /// * otherwise, default to `.` (flat layout)
-    ///
-    /// Additionally, if a `./python` directory exists and is not a package (i.e. it does not contain an `__init__.py` or `__init__.pyi` file),
-    /// it will also be included in the first party search path.
+    /// * `./src`
+    /// * `./<project-name>` (if a `./<project-name>/<project-name>` directory exists)
+    /// * `./python`
     #[serde(skip_serializing_if = "Option::is_none")]
     #[option(
         default = r#"null"#,
@@ -651,11 +683,12 @@ pub struct EnvironmentOptions {
     /// ty uses the `site-packages` directory of your project's Python environment
     /// to resolve third-party (and, in some cases, first-party) imports in your code.
     ///
-    /// If you're using a project management tool such as uv, you should not generally need
-    /// to specify this option, as commands such as `uv run` will set the `VIRTUAL_ENV`
-    /// environment variable to point to your project's virtual environment. ty can also infer
-    /// the location of your environment from an activated Conda environment, and will look for
-    /// a `.venv` directory in the project root if none of the above apply.
+    /// If you're using a project management tool such as uv, you should not generally need to
+    /// specify this option, as commands such as `uv run` will set the `VIRTUAL_ENV` environment
+    /// variable to point to your project's virtual environment. ty can also infer the location of
+    /// your environment from an activated Conda environment, and will look for a `.venv` directory
+    /// in the project root if none of the above apply. Failing that, ty will look for a `python3`
+    /// or `python` binary available in `PATH`.
     ///
     /// Passing a path to a Python executable is supported, but passing a path to a dynamic executable
     /// (such as a shim) is not currently supported.
@@ -689,14 +722,13 @@ pub struct EnvironmentOptions {
 pub struct SrcOptions {
     /// The root of the project, used for finding first-party modules.
     ///
-    /// If left unspecified, ty will try to detect common project layouts and initialize `src.root` accordingly:
+    /// If left unspecified, ty will try to detect common project layouts and initialize `src.root` accordingly.
+    /// The project root (`.`) is always included. Additionally, the following directories are included
+    /// if they exist and are not packages (i.e. they do not contain `__init__.py` or `__init__.pyi` files):
     ///
-    /// * if a `./src` directory exists, include `.` and `./src` in the first party search path (src layout or flat)
-    /// * if a `./<project-name>/<project-name>` directory exists, include `.` and `./<project-name>` in the first party search path
-    /// * otherwise, default to `.` (flat layout)
-    ///
-    /// Additionally, if a `./python` directory exists and is not a package (i.e. it does not contain an `__init__.py` file),
-    /// it will also be included in the first party search path.
+    /// * `./src`
+    /// * `./<project-name>` (if a `./<project-name>/<project-name>` directory exists)
+    /// * `./python`
     #[serde(skip_serializing_if = "Option::is_none")]
     #[option(
         default = r#"null"#,
@@ -877,18 +909,31 @@ impl Rules {
 
         for (rule_name, level) in &self.inner {
             let source = rule_name.source();
+            let lint_source = match source {
+                ValueSource::File(_) => LintSource::File,
+                ValueSource::Cli => LintSource::Cli,
+                ValueSource::Editor => LintSource::Editor,
+            };
+
+            let mut set_lint_level = |lint| {
+                if let Ok(severity) = Severity::try_from(**level) {
+                    selection.enable(lint, severity, lint_source);
+                } else {
+                    selection.disable(lint);
+                }
+            };
+
+            // Handle "all" as a special case - apply the level to all rules
+            if rule_name.eq_ignore_ascii_case("all") {
+                for lint in registry.lints() {
+                    set_lint_level(*lint);
+                }
+                continue;
+            }
+
             match registry.get(rule_name) {
                 Ok(lint) => {
-                    let lint_source = match source {
-                        ValueSource::File(_) => LintSource::File,
-                        ValueSource::Cli => LintSource::Cli,
-                        ValueSource::Editor => LintSource::Editor,
-                    };
-                    if let Ok(severity) = Severity::try_from(**level) {
-                        selection.enable(lint, severity, lint_source);
-                    } else {
-                        selection.disable(lint);
-                    }
+                    set_lint_level(lint);
                 }
                 Err(error) => {
                     // `system_path_to_file` can return `Err` if the file was deleted since the configuration
@@ -986,7 +1031,8 @@ fn build_include_filter(
         }
 
         for pattern in include_patterns {
-            pattern.absolute(project_root, system, PortableGlobKind::Include)
+            pattern
+                .absolute(project_root, system, PortableGlobKind::Include)
                 .and_then(|include| Ok(includes.add(&include)?))
                 .map_err(|err| {
                     let diagnostic = OptionDiagnostic::new(
@@ -995,36 +1041,13 @@ fn build_include_filter(
                         Severity::Error,
                     );
 
-                    match pattern.source() {
-                        ValueSource::File(file_path) => {
-                            if let Ok(file) = system_path_to_file(db, &**file_path) {
-                                diagnostic
-                                    .with_message("Invalid include pattern")
-                                    .with_annotation(Some(
-                                        Annotation::primary(
-                                            Span::from(file)
-                                                .with_optional_range(pattern.range()),
-                                        )
-                                            .message(err.to_string()),
-                                    ))
-                            } else {
-                                diagnostic.sub(SubDiagnostic::new(
-                                    SubDiagnosticSeverity::Info,
-                                    format!("The pattern is defined in the `{}` option in your configuration file", context.include_name()),
-                                ))
-                            }
-                        }
-                        ValueSource::Cli => diagnostic.sub(SubDiagnostic::new(
-                            SubDiagnosticSeverity::Info,
-                            "The pattern was specified on the CLI",
-                        )),
-                        ValueSource::Editor => {
-                            diagnostic.sub(SubDiagnostic::new(
-                                SubDiagnosticSeverity::Info,
-                                "The pattern was specified in the editor settings.",
-                            ))
-                        }
-                    }
+                    diagnostic.with_source_sub(
+                        db,
+                        pattern.value(),
+                        "pattern",
+                        context.include_name(),
+                        err,
+                    )
                 })?;
         }
     } else {
@@ -1074,7 +1097,8 @@ fn build_exclude_filter(
     // Add user-specified excludes
     if let Some(exclude_patterns) = exclude_patterns {
         for exclude in exclude_patterns {
-            exclude.absolute(project_root, system, PortableGlobKind::Exclude)
+            exclude
+                .absolute(project_root, system, PortableGlobKind::Exclude)
                 .and_then(|pattern| Ok(excludes.add(&pattern)?))
                 .map_err(|err| {
                     let diagnostic = OptionDiagnostic::new(
@@ -1083,34 +1107,13 @@ fn build_exclude_filter(
                         Severity::Error,
                     );
 
-                    match exclude.source() {
-                        ValueSource::File(file_path) => {
-                            if let Ok(file) = system_path_to_file(db, &**file_path) {
-                                diagnostic
-                                    .with_message("Invalid exclude pattern")
-                                    .with_annotation(Some(
-                                        Annotation::primary(
-                                            Span::from(file)
-                                                .with_optional_range(exclude.range()),
-                                        )
-                                            .message(err.to_string()),
-                                    ))
-                            } else {
-                                diagnostic.sub(SubDiagnostic::new(
-                                    SubDiagnosticSeverity::Info,
-                                    format!("The pattern is defined in the `{}` option in your configuration file", context.exclude_name()),
-                                ))
-                            }
-                        }
-                        ValueSource::Cli => diagnostic.sub(SubDiagnostic::new(
-                            SubDiagnosticSeverity::Info,
-                            "The pattern was specified on the CLI",
-                        )),
-                        ValueSource::Editor => diagnostic.sub(SubDiagnostic::new(
-                            SubDiagnosticSeverity::Info,
-                            "The pattern was specified in the editor settings",
-                        ))
-                    }
+                    diagnostic.with_source_sub(
+                        db,
+                        exclude.value(),
+                        "pattern",
+                        context.exclude_name(),
+                        err,
+                    )
                 })?;
         }
     }
@@ -1185,6 +1188,9 @@ pub enum OutputFormat {
     ///
     /// [GitHub Actions]: https://docs.github.com/en/actions/reference/workflows-and-actions/workflow-commands#setting-an-error-message
     Github,
+    /// Print diagnostics as a JUnit-style XML report.
+    #[cfg(feature = "junit")]
+    Junit,
 }
 
 impl OutputFormat {
@@ -1205,6 +1211,8 @@ impl From<OutputFormat> for DiagnosticFormat {
             OutputFormat::Concise => Self::Concise,
             OutputFormat::Gitlab => Self::Gitlab,
             OutputFormat::Github => Self::Github,
+            #[cfg(feature = "junit")]
+            OutputFormat::Junit => Self::Junit,
         }
     }
 }
@@ -1240,7 +1248,7 @@ pub struct TerminalOptions {
     #[serde(skip_serializing_if = "Option::is_none")]
     #[option(
         default = r#"full"#,
-        value_type = "full | concise",
+        value_type = "full | concise | github | gitlab | junit",
         example = r#"
             output-format = "concise"
         "#
@@ -1266,6 +1274,7 @@ pub struct TerminalOptions {
     Clone,
     Eq,
     PartialEq,
+    Hash,
     Combine,
     Serialize,
     Deserialize,
@@ -1293,22 +1302,102 @@ pub struct AnalysisOptions {
         "#
     )]
     pub respect_type_ignore_comments: Option<bool>,
+
+    /// A list of module glob patterns for which `unresolved-import` diagnostics should be suppressed.
+    ///
+    /// Details on supported glob patterns:
+    /// - `*` matches zero or more characters except `.`. For example, `foo.*` matches `foo.bar` but
+    ///   not `foo.bar.baz`; `foo*` matches `foo` and `foobar` but not `foo.bar` or `barfoo`; and `*foo`
+    ///   matches `foo` and `barfoo` but not `foo.bar` or `foobar`.
+    /// - `**` matches any number of module components (e.g., `foo.**` matches `foo`, `foo.bar`, etc.)
+    /// - Prefix a pattern with `!` to exclude matching modules
+    ///
+    /// When multiple patterns match, later entries take precedence.
+    ///
+    /// Glob patterns can be used in combinations with each other. For example, to suppress errors for
+    /// any module where the first component contains the substring `test`, use `*test*.**`.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    #[option(
+        default = r#"[]"#,
+        value_type = "list[str]",
+        example = r#"
+            # Suppress errors for all `test` modules except `test.foo`
+            allowed-unresolved-imports = ["test.**", "!test.foo"]
+        "#
+    )]
+    pub allowed_unresolved_imports: Option<Vec<RangedValue<String>>>,
 }
 
 impl AnalysisOptions {
-    fn to_settings(&self) -> AnalysisSettings {
+    pub(super) fn to_settings(
+        &self,
+        db: &dyn Db,
+        diagnostics: &mut Vec<OptionDiagnostic>,
+    ) -> AnalysisSettings {
+        let Self {
+            respect_type_ignore_comments,
+            allowed_unresolved_imports,
+        } = self;
+
         let AnalysisSettings {
             respect_type_ignore_comments: respect_type_ignore_default,
+            allowed_unresolved_imports: allowed_unresolved_imports_default,
         } = AnalysisSettings::default();
 
+        let allowed_unresolved_imports =
+            if let Some(allowed_unresolved_imports) = allowed_unresolved_imports {
+                build_module_glob_set(db, allowed_unresolved_imports, "allowed_unresolved_imports")
+                    .unwrap_or_else(|error| {
+                        diagnostics.push(*error);
+                        ModuleGlobSet::empty()
+                    })
+            } else {
+                allowed_unresolved_imports_default
+            };
+
         AnalysisSettings {
-            respect_type_ignore_comments: self
-                .respect_type_ignore_comments
+            respect_type_ignore_comments: respect_type_ignore_comments
                 .unwrap_or(respect_type_ignore_default),
+            allowed_unresolved_imports,
         }
     }
 }
 
+fn build_module_glob_set(
+    db: &dyn Db,
+    patterns: &[RangedValue<String>],
+    option_name: &str,
+) -> Result<ModuleGlobSet, Box<OptionDiagnostic>> {
+    let mut builder = ModuleGlobSetBuilder::new();
+
+    for glob in patterns {
+        if let Err(error) = builder.add(glob) {
+            let diagnostic = OptionDiagnostic::new(
+                DiagnosticId::InvalidGlob,
+                format!("Invalid glob pattern `{error}`"),
+                Severity::Error,
+            );
+
+            return Err(diagnostic
+                .with_source_sub(db, glob, "glob", option_name, error)
+                .into());
+        }
+    }
+
+    builder.build().map_err(|_| {
+        let diagnostic = OptionDiagnostic::new(
+            DiagnosticId::InvalidGlob,
+            "The `{option_name}` patterns resulted in a regex that is too large".to_string(),
+            Severity::Error,
+        );
+
+        Box::new(diagnostic.sub(SubDiagnostic::new(
+            SubDiagnosticSeverity::Info,
+            "Please open an issue on the ty repository and share the patterns that caused the error.",
+        )))
+    })
+}
+
 /// Configuration override that applies to specific files based on glob patterns.
 ///
 /// An override allows you to apply different rule configurations to specific
@@ -1436,7 +1525,7 @@ pub struct OverrideOptions {
     #[serde(skip_serializing_if = "Option::is_none")]
     #[option(
         default = r#"{...}"#,
-        value_type = r#"dict[RuleName, "ignore" | "warn" | "error"]"#,
+        value_type = r#"dict[RuleName | "all", "ignore" | "warn" | "error"]"#,
         example = r#"
             [[tool.ty.overrides]]
             include = ["src"]
@@ -1446,6 +1535,10 @@ pub struct OverrideOptions {
         "#
     )]
     pub rules: Option<Rules>,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    #[option_group]
+    pub analysis: Option<AnalysisOptions>,
 }
 
 impl RangedValue<OverrideOptions> {
@@ -1454,36 +1547,49 @@ impl RangedValue<OverrideOptions> {
         db: &dyn Db,
         project_root: &SystemPath,
         global_rules: Option<&Rules>,
+        global_analysis: Option<&AnalysisOptions>,
         diagnostics: &mut Vec<OptionDiagnostic>,
     ) -> Result<Option<Override>, Box<OptionDiagnostic>> {
         let rules = self.rules.or_default();
+        let analysis = self.analysis.or_default();
 
         // First, warn about incorrect or useless overrides.
-        if rules.is_empty() {
+        if rules.is_empty() && *analysis == AnalysisOptions::default() {
             let mut diagnostic = OptionDiagnostic::new(
                 DiagnosticId::UselessOverridesSection,
                 "Useless `overrides` section".to_string(),
                 Severity::Warning,
             );
 
-            diagnostic = if self.rules.is_none() {
+            diagnostic = if self.rules.is_none() && self.analysis.is_none() {
                 diagnostic = diagnostic.sub(SubDiagnostic::new(
                     SubDiagnosticSeverity::Info,
-                    "It has no `rules` table",
+                    "It has no `rules` or `analysis` table",
                 ));
                 diagnostic.sub(SubDiagnostic::new(
                     SubDiagnosticSeverity::Info,
-                    "Add a `[overrides.rules]` table...",
+                    "Add a `[overrides.rules]` or `[overrides.analysis]` table...",
                 ))
             } else {
-                diagnostic = diagnostic.sub(SubDiagnostic::new(
-                    SubDiagnosticSeverity::Info,
-                    "The rules table is empty",
-                ));
-                diagnostic.sub(SubDiagnostic::new(
-                    SubDiagnosticSeverity::Info,
-                    "Add a rule to `[overrides.rules]` to override specific rules...",
-                ))
+                if self.rules.is_some() && rules.is_empty() {
+                    diagnostic = diagnostic.sub(SubDiagnostic::new(
+                        SubDiagnosticSeverity::Info,
+                        "The `rules` table is empty",
+                    ));
+                    diagnostic = diagnostic.sub(SubDiagnostic::new(
+                        SubDiagnosticSeverity::Info,
+                        "Add a rule to `[overrides.rules]` to override specific rules...",
+                    ));
+                }
+
+                if self.analysis.is_some() && *analysis == AnalysisOptions::default() {
+                    diagnostic = diagnostic.sub(SubDiagnostic::new(
+                        SubDiagnosticSeverity::Info,
+                        "The `analysis` table is empty",
+                    ));
+                }
+
+                diagnostic
             };
 
             diagnostic = diagnostic.sub(SubDiagnostic::new(
@@ -1496,7 +1602,7 @@ impl RangedValue<OverrideOptions> {
                 if let Ok(file) = system_path_to_file(db, source_file) {
                     let annotation =
                         Annotation::primary(Span::from(file).with_optional_range(self.range()))
-                            .message("This overrides section configures no rules");
+                            .message("This overrides section overrides no settings");
                     diagnostic = diagnostic.with_annotation(Some(annotation));
                 }
             }
@@ -1585,13 +1691,23 @@ impl RangedValue<OverrideOptions> {
         // Convert merged rules to rule selection
         let rule_selection = merged_rules.to_rule_selection(db, diagnostics);
 
+        let mut merged_analysis = analysis.into_owned();
+
+        if let Some(global_analysis) = global_analysis {
+            merged_analysis = merged_analysis.combine(global_analysis.clone());
+        }
+
+        let analysis = merged_analysis.to_settings(db, diagnostics);
+
         let override_instance = Override {
             files,
             options: Arc::new(InnerOverrideOptions {
                 rules: self.rules.clone(),
+                analysis: self.analysis.clone(),
             }),
             settings: Arc::new(OverrideSettings {
                 rules: rule_selection,
+                analysis,
             }),
         };
 
@@ -1605,6 +1721,8 @@ pub(super) struct InnerOverrideOptions {
     /// Raw rule options as specified in the configuration.
     /// Used when multiple overrides match a file and need to be merged.
     pub(super) rules: Option<Rules>,
+
+    pub(super) analysis: Option<AnalysisOptions>,
 }
 
 /// Error returned when the settings can't be resolved because of a hard error.
@@ -1624,7 +1742,7 @@ impl ToSettingsError {
 
         impl fmt::Display for DisplayPretty<'_> {
             fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-                let display_config = DisplayDiagnosticConfig::default()
+                let display_config = DisplayDiagnosticConfig::new("ty")
                     .format(self.error.output_format.into())
                     .color(self.error.color);
 
@@ -1720,6 +1838,7 @@ pub enum TyTomlError {
 pub struct OptionDiagnostic {
     id: DiagnosticId,
     message: String,
+    concise_message: Option<String>,
     severity: Severity,
     annotation: Option<Annotation>,
     sub: Vec<SubDiagnostic>,
@@ -1730,6 +1849,7 @@ impl OptionDiagnostic {
         Self {
             id,
             message,
+            concise_message: None,
             severity,
             annotation: None,
             sub: Vec::new(),
@@ -1744,11 +1864,60 @@ impl OptionDiagnostic {
         }
     }
 
+    #[must_use]
+    fn with_concise_message(self, message: impl Display) -> Self {
+        OptionDiagnostic {
+            concise_message: Some(message.to_string()),
+            ..self
+        }
+    }
+
     #[must_use]
     fn with_annotation(self, annotation: Option<Annotation>) -> Self {
         OptionDiagnostic { annotation, ..self }
     }
 
+    fn with_source_sub<T>(
+        mut self,
+        db: &dyn Db,
+        value: &RangedValue<T>,
+        value_label: &str,
+        option_name: &str,
+        err: impl Display,
+    ) -> Self {
+        match value.source() {
+            ValueSource::File(file_path) => {
+                if let Ok(file) = system_path_to_file(db, &**file_path) {
+                    let concise_message = std::mem::take(&mut self.message);
+                    self.with_concise_message(concise_message)
+                        .with_message(format_args!("Invalid {value_label}"))
+                        .with_annotation(Some(
+                            Annotation::primary(
+                                Span::from(file).with_optional_range(value.range()),
+                            )
+                            .message(err.to_string()),
+                        ))
+                } else {
+                    self.sub(SubDiagnostic::new(
+                        SubDiagnosticSeverity::Info,
+                        format!(
+                            "The {value_label} is defined in the `{option_name}` option \
+                            in your configuration file"
+                        ),
+                    ))
+                }
+            }
+            ValueSource::Cli => self.sub(SubDiagnostic::new(
+                SubDiagnosticSeverity::Info,
+                "The {value_label} was specified on the CLI",
+            )),
+            ValueSource::Editor => self.sub(SubDiagnostic::new(
+                SubDiagnosticSeverity::Info,
+                "The {value_label} was specified in the editor settings.",
+            )),
+        }
+    }
+
     #[must_use]
     fn sub(mut self, sub: SubDiagnostic) -> Self {
         self.sub.push(sub);
@@ -1757,6 +1926,11 @@ impl OptionDiagnostic {
 
     pub(crate) fn to_diagnostic(&self) -> Diagnostic {
         let mut diag = Diagnostic::new(self.id, self.severity, &self.message);
+
+        if let Some(concise_message) = &self.concise_message {
+            diag.set_concise_message(concise_message);
+        }
+
         if let Some(annotation) = self.annotation.clone() {
             diag.annotate(annotation);
         }
