@@ -1,34 +1,64 @@
 use ruff_db::files::{File, FilePath};
-use ruff_db::source::line_index;
-use ruff_python_ast as ast;
+use ruff_db::parsed::{parsed_module, parsed_string_annotation};
+use ruff_db::source::{line_index, source_text};
+use ruff_python_ast::{self as ast, ExprStringLiteral, ModExpression};
 use ruff_python_ast::{Expr, ExprRef, HasNodeIndex, name::Name};
+use ruff_python_parser::Parsed;
 use ruff_source_file::LineIndex;
+use rustc_hash::FxHashMap;
+use ty_module_resolver::{
+    KnownModule, Module, ModuleName, list_modules, resolve_module, resolve_real_shadowable_module,
+};
 
 use crate::Db;
-use crate::module_name::ModuleName;
-use crate::module_resolver::{KnownModule, Module, list_modules, resolve_module};
+use crate::place::implicit_globals::all_implicit_module_globals;
 use crate::semantic_index::definition::Definition;
+use crate::semantic_index::place_table;
 use crate::semantic_index::scope::FileScopeId;
 use crate::semantic_index::semantic_index;
-use crate::types::ide_support::all_declarations_and_bindings;
-use crate::types::{Type, binding_type, infer_scope_types};
+use crate::semantic_index::symbol::Symbol;
+use crate::types::ide_support::{ImportAliasResolution, definition_for_name};
+use crate::types::list_members::{Member, all_members, all_reachable_members};
+use crate::types::{
+    Type, TypeQualifiers, binding_type, declaration_type, infer_complete_scope_types,
+};
 
+/// The primary interface the LSP should use for querying semantic information about a [`File`].
+///
+/// Although you can in principle freely construct this type given a `db` and `file`, you should
+/// try to construct this at the start of your analysis and thread the same instance through
+/// the full analysis.
+///
+/// The primary reason for this is that it manages traversing into the sub-ASTs of string
+/// annotations (see [`Self::enter_string_annotation`]). When you do this you will be handling
+/// AST nodes that don't belong to the file's AST (or *any* file's AST). These kinds of nodes
+/// will result in panics and confusing results if handed to the wrong subsystem. `SemanticModel`
+/// methods will automatically handle using the string literal's AST node when necessary.
 pub struct SemanticModel<'db> {
     db: &'db dyn Db,
     file: File,
+    /// If `Some` then this `SemanticModel` is for analyzing the sub-AST of a string annotation.
+    /// This expression will be used as a witness to the scope/location we're analyzing.
+    in_string_annotation_expr: Option<Box<Expr>>,
 }
 
 impl<'db> SemanticModel<'db> {
     pub fn new(db: &'db dyn Db, file: File) -> Self {
-        Self { db, file }
+        Self {
+            db,
+            file,
+            in_string_annotation_expr: None,
+        }
     }
 
-    // TODO we don't actually want to expose the Db directly to lint rules, but we need to find a
-    // solution for exposing information from types
-    pub fn db(&self) -> &dyn Db {
+    pub fn db(&self) -> &'db dyn Db {
         self.db
     }
 
+    pub fn file(&self) -> File {
+        self.file
+    }
+
     pub fn file_path(&self) -> &FilePath {
         self.file.path(self.db)
     }
@@ -37,21 +67,66 @@ impl<'db> SemanticModel<'db> {
         line_index(self.db, self.file)
     }
 
-    pub fn resolve_module(&self, module_name: &ModuleName) -> Option<Module<'_>> {
-        resolve_module(self.db, module_name)
+    /// Returns a map from symbol name to that symbol's
+    /// type and definition site (if available).
+    ///
+    /// The symbols are the symbols in scope at the given
+    /// AST node.
+    pub fn members_in_scope_at(
+        &self,
+        node: ast::AnyNodeRef<'_>,
+    ) -> FxHashMap<Name, MemberDefinition<'db>> {
+        let index = semantic_index(self.db, self.file);
+        let mut members = FxHashMap::default();
+        let Some(file_scope) = self.scope(node) else {
+            return members;
+        };
+
+        for (file_scope, _) in index.ancestor_scopes(file_scope) {
+            for memberdef in
+                all_reachable_members(self.db, file_scope.to_scope_id(self.db, self.file))
+            {
+                members.insert(
+                    memberdef.member.name,
+                    MemberDefinition {
+                        ty: memberdef.member.ty,
+                        first_reachable_definition: memberdef.first_reachable_definition,
+                    },
+                );
+            }
+        }
+        members
+    }
+
+    /// Resolve the given import made in this file to a Type
+    pub fn resolve_module_type(&self, module: Option<&str>, level: u32) -> Option<Type<'db>> {
+        let module = self.resolve_module(module, level)?;
+        Some(Type::module_literal(self.db, self.file, module))
+    }
+
+    /// Resolve the given import made in this file to a Module
+    pub fn resolve_module(&self, module: Option<&str>, level: u32) -> Option<Module<'db>> {
+        let module_name =
+            ModuleName::from_identifier_parts(self.db, self.file, module, level).ok()?;
+        resolve_module(self.db, self.file, &module_name)
     }
 
     /// Returns completions for symbols available in a `import <CURSOR>` context.
     pub fn import_completions(&self) -> Vec<Completion<'db>> {
+        let typing_extensions = ModuleName::new_static("typing_extensions").unwrap();
+        let is_typing_extensions_available = self.file.is_stub(self.db)
+            || resolve_real_shadowable_module(self.db, self.file, &typing_extensions).is_some();
         list_modules(self.db)
             .into_iter()
+            .filter(|module| {
+                is_typing_extensions_available || module.name(self.db) != &typing_extensions
+            })
             .map(|module| {
                 let builtin = module.is_known(self.db, KnownModule::Builtins);
                 let ty = Type::module_literal(self.db, self.file, module);
                 Completion {
                     name: Name::new(module.name(self.db).as_str()),
                     ty: Some(ty),
-                    kind: None,
                     builtin,
                 }
             })
@@ -59,11 +134,7 @@ impl<'db> SemanticModel<'db> {
     }
 
     /// Returns completions for symbols available in a `from module import <CURSOR>` context.
-    pub fn from_import_completions(
-        &self,
-        import: &ast::StmtImportFrom,
-        _name: Option<usize>,
-    ) -> Vec<Completion<'db>> {
+    pub fn from_import_completions(&self, import: &ast::StmtImportFrom) -> Vec<Completion<'db>> {
         let module_name = match ModuleName::from_import_statement(self.db, self.file, import) {
             Ok(module_name) => module_name,
             Err(err) => {
@@ -78,73 +149,12 @@ impl<'db> SemanticModel<'db> {
         self.module_completions(&module_name)
     }
 
-    /// Returns completions only for submodules for the module
-    /// identified by `name` in `import`.
-    ///
-    /// For example, `import re, os.<CURSOR>, zlib`.
-    pub fn import_submodule_completions(
-        &self,
-        import: &ast::StmtImport,
-        name: usize,
-    ) -> Vec<Completion<'db>> {
-        let module_ident = &import.names[name].name;
-        let Some((parent_ident, _)) = module_ident.rsplit_once('.') else {
-            return vec![];
-        };
-        let module_name =
-            match ModuleName::from_identifier_parts(self.db, self.file, Some(parent_ident), 0) {
-                Ok(module_name) => module_name,
-                Err(err) => {
-                    tracing::debug!(
-                        "Could not extract module name from `{module:?}`: {err:?}",
-                        module = module_ident,
-                    );
-                    return vec![];
-                }
-            };
-        self.import_submodule_completions_for_name(&module_name)
-    }
-
-    /// Returns completions only for submodules for the module
-    /// used in a `from module import attribute` statement.
-    ///
-    /// For example, `from os.<CURSOR>`.
-    pub fn from_import_submodule_completions(
-        &self,
-        import: &ast::StmtImportFrom,
-    ) -> Vec<Completion<'db>> {
-        let level = import.level;
-        let Some(module_ident) = import.module.as_deref() else {
-            return vec![];
-        };
-        let Some((parent_ident, _)) = module_ident.rsplit_once('.') else {
-            return vec![];
-        };
-        let module_name = match ModuleName::from_identifier_parts(
-            self.db,
-            self.file,
-            Some(parent_ident),
-            level,
-        ) {
-            Ok(module_name) => module_name,
-            Err(err) => {
-                tracing::debug!(
-                    "Could not extract module name from `{module:?}` with level {level}: {err:?}",
-                    module = import.module,
-                    level = import.level,
-                );
-                return vec![];
-            }
-        };
-        self.import_submodule_completions_for_name(&module_name)
-    }
-
     /// Returns submodule-only completions for the given module.
-    fn import_submodule_completions_for_name(
+    pub fn import_submodule_completions_for_name(
         &self,
         module_name: &ModuleName,
     ) -> Vec<Completion<'db>> {
-        let Some(module) = resolve_module(self.db, module_name) else {
+        let Some(module) = resolve_module(self.db, self.file, module_name) else {
             tracing::debug!("Could not resolve module from `{module_name:?}`");
             return vec![];
         };
@@ -154,7 +164,7 @@ impl<'db> SemanticModel<'db> {
     /// Returns completions for symbols available in the given module as if
     /// it were imported by this model's `File`.
     fn module_completions(&self, module_name: &ModuleName) -> Vec<Completion<'db>> {
-        let Some(module) = resolve_module(self.db, module_name) else {
+        let Some(module) = resolve_module(self.db, self.file, module_name) else {
             tracing::debug!("Could not resolve module from `{module_name:?}`");
             return vec![];
         };
@@ -162,11 +172,10 @@ impl<'db> SemanticModel<'db> {
         let builtin = module.is_known(self.db, KnownModule::Builtins);
 
         let mut completions = vec![];
-        for crate::types::Member { name, ty } in crate::types::all_members(self.db, ty) {
+        for Member { name, ty } in all_members(self.db, ty) {
             completions.push(Completion {
                 name,
                 ty: Some(ty),
-                kind: None,
                 builtin,
             });
         }
@@ -181,10 +190,12 @@ impl<'db> SemanticModel<'db> {
         let mut completions = vec![];
         for submodule in module.all_submodules(self.db) {
             let ty = Type::module_literal(self.db, self.file, *submodule);
+            let Some(base) = submodule.name(self.db).components().next_back() else {
+                continue;
+            };
             completions.push(Completion {
-                name: Name::new(submodule.name(self.db).as_str()),
+                name: Name::new(base),
                 ty: Some(ty),
-                kind: None,
                 builtin,
             });
         }
@@ -193,13 +204,15 @@ impl<'db> SemanticModel<'db> {
 
     /// Returns completions for symbols available in a `object.<CURSOR>` context.
     pub fn attribute_completions(&self, node: &ast::ExprAttribute) -> Vec<Completion<'db>> {
-        let ty = node.value.inferred_type(self);
-        crate::types::all_members(self.db, ty)
+        let Some(ty) = node.value.inferred_type(self) else {
+            return Vec::new();
+        };
+
+        all_members(self.db, ty)
             .into_iter()
             .map(|member| Completion {
                 name: member.name,
                 ty: Some(member.ty),
-                kind: None,
                 builtin: false,
             })
             .collect()
@@ -213,12 +226,95 @@ impl<'db> SemanticModel<'db> {
     pub fn scoped_completions(&self, node: ast::AnyNodeRef<'_>) -> Vec<Completion<'db>> {
         let index = semantic_index(self.db, self.file);
 
-        // TODO: We currently use `try_expression_scope_id` here as a hotfix for [1].
-        // Revert this to use `expression_scope_id` once a proper fix is in place.
-        //
-        // [1] https://github.com/astral-sh/ty/issues/572
-        let Some(file_scope) = (match node {
+        let Some(file_scope) = self.scope(node) else {
+            return vec![];
+        };
+        let mut completions = vec![];
+        for (file_scope, _) in index.ancestor_scopes(file_scope) {
+            completions.extend(
+                all_reachable_members(self.db, file_scope.to_scope_id(self.db, self.file)).map(
+                    |memberdef| Completion {
+                        name: memberdef.member.name,
+                        ty: Some(memberdef.member.ty),
+                        builtin: false,
+                    },
+                ),
+            );
+        }
+
+        // Add implicit module globals (like `__file__`, `__name__`, etc.) with their
+        // correct types. These are added before builtins so that the deduplication
+        // keeps the correct types (e.g., `__file__` is `str` for the current module,
+        // not `str | None`).
+        completions.extend(
+            all_implicit_module_globals(self.db).map(|(name, ty)| Completion {
+                name,
+                ty: Some(ty),
+                builtin: true,
+            }),
+        );
+
+        // Builtins are available in all scopes.
+        let builtins = ModuleName::new_static("builtins").expect("valid module name");
+        completions.extend(self.module_completions(&builtins));
+
+        // The above can sometimes result in duplicates. Get rid of them.
+        completions.sort_by(|c1, c2| c1.name.cmp(&c2.name));
+        completions.dedup_by(|c1, c2| c1.name == c2.name);
+
+        completions
+    }
+
+    /// Returns `true` if the given class definition's name was previously
+    /// bound in the same scope (i.e., the class definition is a re-assignment).
+    pub fn is_class_name_reassigned(&self, class_def: &ast::StmtClassDef) -> bool {
+        let index = semantic_index(self.db, self.file);
+        let definition = index.expect_single_definition(class_def);
+        let scope = definition.scope(self.db);
+        let table = place_table(self.db, scope);
+        let place = table.place(definition.place(self.db));
+        place.as_symbol().is_some_and(Symbol::is_reassigned)
+    }
+
+    /// Returns the scope in which `node` is defined (handles string annotations).
+    pub fn scope(&self, node: ast::AnyNodeRef<'_>) -> Option<FileScopeId> {
+        let index = semantic_index(self.db, self.file);
+        match self.node_in_ast(node) {
             ast::AnyNodeRef::Identifier(identifier) => index.try_expression_scope_id(identifier),
+
+            // Nodes implementing `HasDefinition`
+            ast::AnyNodeRef::StmtFunctionDef(function) => Some(
+                function
+                    .definition(self)
+                    .scope(self.db)
+                    .file_scope_id(self.db),
+            ),
+            ast::AnyNodeRef::StmtClassDef(class) => {
+                Some(class.definition(self).scope(self.db).file_scope_id(self.db))
+            }
+            ast::AnyNodeRef::Parameter(parameter) => Some(
+                parameter
+                    .definition(self)
+                    .scope(self.db)
+                    .file_scope_id(self.db),
+            ),
+            ast::AnyNodeRef::ParameterWithDefault(parameter) => Some(
+                parameter
+                    .definition(self)
+                    .scope(self.db)
+                    .file_scope_id(self.db),
+            ),
+            ast::AnyNodeRef::ExceptHandlerExceptHandler(handler) => Some(
+                handler
+                    .definition(self)
+                    .scope(self.db)
+                    .file_scope_id(self.db),
+            ),
+            ast::AnyNodeRef::TypeParamTypeVar(var) => {
+                Some(var.definition(self).scope(self.db).file_scope_id(self.db))
+            }
+
+            // Fallback
             node => match node.as_expr_ref() {
                 // If we couldn't identify a specific
                 // expression that we're in, then just
@@ -226,26 +322,129 @@ impl<'db> SemanticModel<'db> {
                 None => Some(FileScopeId::global()),
                 Some(expr) => index.try_expression_scope_id(&expr),
             },
-        }) else {
-            return vec![];
-        };
-        let mut completions = vec![];
-        for (file_scope, _) in index.ancestor_scopes(file_scope) {
-            completions.extend(
-                all_declarations_and_bindings(self.db, file_scope.to_scope_id(self.db, self.file))
-                    .map(|member| Completion {
-                        name: member.name,
-                        ty: Some(member.ty),
-                        kind: None,
-                        builtin: false,
-                    }),
-            );
         }
-        // Builtins are available in all scopes.
-        let builtins = ModuleName::new("builtins").expect("valid module name");
-        completions.extend(self.module_completions(&builtins));
-        completions
     }
+
+    /// Get a "safe" [`ast::AnyNodeRef`] to use for referring to the given (sub-)AST node.
+    ///
+    /// If we're analyzing a string annotation, it will return the string literal's node.
+    /// Otherwise it will return the input.
+    pub fn node_in_ast<'a>(&'a self, node: ast::AnyNodeRef<'a>) -> ast::AnyNodeRef<'a> {
+        if let Some(string_annotation) = &self.in_string_annotation_expr {
+            (&**string_annotation).into()
+        } else {
+            node
+        }
+    }
+
+    /// Get a "safe" [`Expr`] to use for referring to the given (sub-)expression.
+    ///
+    /// If we're analyzing a string annotation, it will return the string literal's expression.
+    /// Otherwise it will return the input.
+    pub fn expr_in_ast<'a>(&'a self, expr: &'a Expr) -> &'a Expr {
+        if let Some(string_annotation) = &self.in_string_annotation_expr {
+            string_annotation
+        } else {
+            expr
+        }
+    }
+
+    /// Get a "safe" [`ExprRef`] to use for referring to the given (sub-)expression.
+    ///
+    /// If we're analyzing a string annotation, it will return the string literal's expression.
+    /// Otherwise it will return the input.
+    pub fn expr_ref_in_ast<'a>(&'a self, expr: ExprRef<'a>) -> ExprRef<'a> {
+        if let Some(string_annotation) = &self.in_string_annotation_expr {
+            ExprRef::from(string_annotation)
+        } else {
+            expr
+        }
+    }
+
+    /// Given a string expression, determine if it's a string annotation, and if it is,
+    /// yield the parsed sub-AST and a sub-model that knows it's analyzing a sub-AST.
+    ///
+    /// Analysis of the sub-AST should only be done with the sub-model, or else things
+    /// may return nonsense results or even panic!
+    pub fn enter_string_annotation(
+        &self,
+        string_expr: &ExprStringLiteral,
+    ) -> Option<(Parsed<ModExpression>, Self)> {
+        // Ask the inference engine whether this is actually a string annotation
+        let expr = ExprRef::StringLiteral(string_expr);
+        let index = semantic_index(self.db, self.file);
+        // When looking up scopes, use the expr in the top-level AST
+        // (we might be trying to enter a sub-sub-AST, so this isn't silly)
+        let file_scope = index.expression_scope_id(&self.expr_ref_in_ast(expr));
+        let scope = file_scope.to_scope_id(self.db, self.file);
+        // When querying whether the expr is a string annotation, we do however use the actual expr
+        // (the inference engine should record this information even for sub-nodes)
+        if !infer_complete_scope_types(self.db, scope).is_string_annotation(expr) {
+            return None;
+        }
+
+        // Parse the sub-AST and create a semantic model that knows it's in a sub-AST
+        //
+        // The string_annotation will be used as the expr/node for any query that needs
+        // to look up a node in the AST to prevent panics, because these sub-AST nodes
+        // are not in the File's AST!
+        let source = source_text(self.db, self.file);
+        let string_literal = string_expr.as_single_part_string()?;
+        let ast = parsed_string_annotation(source.as_str(), string_literal).ok()?;
+        let model = Self {
+            db: self.db,
+            file: self.file,
+            // Use expr_in_ast here because we might be entering a sub-sub-AST
+            in_string_annotation_expr: Some(Box::new(
+                self.expr_in_ast(&Expr::StringLiteral(string_expr.clone()))
+                    .clone(),
+            )),
+        };
+        Some((ast, model))
+    }
+
+    /// Returns the type qualifiers (e.g. `Final`, `ClassVar`) for a given expression,
+    /// if the expression refers to a name or attribute with declared qualifiers.
+    pub fn type_qualifiers(&self, expr: ExprRef<'_>) -> TypeQualifiers {
+        match expr {
+            ExprRef::Name(name) => {
+                let Some(definition) =
+                    definition_for_name(self, name, ImportAliasResolution::ResolveAliases)
+                else {
+                    return TypeQualifiers::empty();
+                };
+                let module = parsed_module(self.db, self.file).load(self.db);
+                if !definition
+                    .kind(self.db)
+                    .category(self.file.is_stub(self.db), &module)
+                    .is_declaration()
+                {
+                    return TypeQualifiers::empty();
+                }
+                declaration_type(self.db, definition).qualifiers()
+            }
+            ExprRef::Attribute(attr) => {
+                let Some(value_ty) = attr.value.inferred_type(self) else {
+                    return TypeQualifiers::empty();
+                };
+                value_ty
+                    .member_lookup_with_policy(
+                        self.db,
+                        attr.attr.id.clone(),
+                        crate::types::MemberLookupPolicy::default(),
+                    )
+                    .qualifiers
+            }
+            _ => TypeQualifiers::empty(),
+        }
+    }
+}
+
+/// The type and definition of a symbol.
+#[derive(Clone, Debug)]
+pub struct MemberDefinition<'db> {
+    pub ty: Type<'db>,
+    pub first_reachable_definition: Definition<'db>,
 }
 
 /// A classification of symbol names.
@@ -256,7 +455,7 @@ impl<'db> SemanticModel<'db> {
 /// single-underscore names. This matches the order of the variants defined for
 /// this enum, which is in turn picked up by the derived trait implementation
 /// for `Ord`.
-#[derive(Clone, Copy, Eq, PartialEq, PartialOrd, Ord)]
+#[derive(Clone, Copy, Debug, Eq, PartialEq, PartialOrd, Ord)]
 pub enum NameKind {
     Normal,
     Dunder,
@@ -293,15 +492,6 @@ pub struct Completion<'db> {
     /// an unimported symbol. In that case, computing the
     /// type of all such symbols could be quite expensive.
     pub ty: Option<Type<'db>>,
-    /// The "kind" of this completion.
-    ///
-    /// When this is set, it takes priority over any kind
-    /// inferred from `ty`.
-    ///
-    /// Usually this is set when `ty` is `None`, since it
-    /// may be cheaper to compute at scale. (e.g., For
-    /// unimported symbol completions.)
-    pub kind: Option<CompletionKind>,
     /// Whether this suggestion came from builtins or not.
     ///
     /// At time of writing (2025-06-26), this information
@@ -311,106 +501,16 @@ pub struct Completion<'db> {
     pub builtin: bool,
 }
 
-impl<'db> Completion<'db> {
-    /// Returns the "kind" of this completion.
-    ///
-    /// This is meant to be a very general classification of this completion.
-    /// Typically, this is communicated from the LSP server to a client, and
-    /// the client uses this information to help improve the UX (perhaps by
-    /// assigning an icon of some kind to the completion).
-    pub fn kind(&self, db: &'db dyn Db) -> Option<CompletionKind> {
-        fn imp<'db>(db: &'db dyn Db, ty: Type<'db>) -> Option<CompletionKind> {
-            Some(match ty {
-                Type::FunctionLiteral(_)
-                | Type::DataclassDecorator(_)
-                | Type::WrapperDescriptor(_)
-                | Type::DataclassTransformer(_)
-                | Type::Callable(_) => CompletionKind::Function,
-                Type::BoundMethod(_) | Type::KnownBoundMethod(_) => CompletionKind::Method,
-                Type::ModuleLiteral(_) => CompletionKind::Module,
-                Type::ClassLiteral(_) | Type::GenericAlias(_) | Type::SubclassOf(_) => {
-                    CompletionKind::Class
-                }
-                // This is a little weird for "struct." I'm mostly interpreting
-                // "struct" here as a more general "object." ---AG
-                Type::NominalInstance(_)
-                | Type::PropertyInstance(_)
-                | Type::BoundSuper(_)
-                | Type::TypedDict(_) => CompletionKind::Struct,
-                Type::IntLiteral(_)
-                | Type::BooleanLiteral(_)
-                | Type::TypeIs(_)
-                | Type::StringLiteral(_)
-                | Type::LiteralString
-                | Type::BytesLiteral(_) => CompletionKind::Value,
-                Type::EnumLiteral(_) => CompletionKind::Enum,
-                Type::ProtocolInstance(_) => CompletionKind::Interface,
-                Type::NonInferableTypeVar(_) | Type::TypeVar(_) => CompletionKind::TypeParameter,
-                Type::Union(union) => union.elements(db).iter().find_map(|&ty| imp(db, ty))?,
-                Type::Intersection(intersection) => {
-                    intersection.iter_positive(db).find_map(|ty| imp(db, ty))?
-                }
-                Type::Dynamic(_)
-                | Type::Never
-                | Type::SpecialForm(_)
-                | Type::KnownInstance(_)
-                | Type::AlwaysTruthy
-                | Type::AlwaysFalsy => return None,
-                Type::TypeAlias(alias) => imp(db, alias.value_type(db))?,
-            })
-        }
-        self.kind.or_else(|| self.ty.and_then(|ty| imp(db, ty)))
-    }
-}
-
-/// The "kind" of a completion.
-///
-/// This is taken directly from the LSP completion specification:
-/// <https://microsoft.github.io/language-server-protocol/specifications/lsp/3.17/specification/#completionItemKind>
-///
-/// The idea here is that `Completion::kind` defines the mapping to this from
-/// `Type` (and possibly other information), which might be interesting and
-/// contentious. Then the outer edges map this to the LSP types, which is
-/// expected to be mundane and boring.
-#[derive(Clone, Copy, Debug)]
-pub enum CompletionKind {
-    Text,
-    Method,
-    Function,
-    Constructor,
-    Field,
-    Variable,
-    Class,
-    Interface,
-    Module,
-    Property,
-    Unit,
-    Value,
-    Enum,
-    Keyword,
-    Snippet,
-    Color,
-    File,
-    Reference,
-    Folder,
-    EnumMember,
-    Constant,
-    Struct,
-    Event,
-    Operator,
-    TypeParameter,
-}
-
 pub trait HasType {
     /// Returns the inferred type of `self`.
     ///
     /// ## Panics
     /// May panic if `self` is from another file than `model`.
-    fn inferred_type<'db>(&self, model: &SemanticModel<'db>) -> Type<'db>;
+    fn inferred_type<'db>(&self, model: &SemanticModel<'db>) -> Option<Type<'db>>;
 }
 
 pub trait HasDefinition {
-    /// Returns the inferred type of `self`.
+    /// Returns the definition of `self`.
     ///
     /// ## Panics
     /// May panic if `self` is from another file than `model`.
@@ -418,12 +518,16 @@ pub trait HasDefinition {
 }
 
 impl HasType for ast::ExprRef<'_> {
-    fn inferred_type<'db>(&self, model: &SemanticModel<'db>) -> Type<'db> {
+    fn inferred_type<'db>(&self, model: &SemanticModel<'db>) -> Option<Type<'db>> {
         let index = semantic_index(model.db, model.file);
-        let file_scope = index.expression_scope_id(self);
+        // TODO(#1637): semantic tokens is making this crash even with
+        // `try_expr_ref_in_ast` guarding this, for now just use `try_expression_scope_id`.
+        // The problematic input is `x: "float` (with a dangling quote). I imagine the issue
+        // is we're too eagerly setting `is_string_annotation` in inference.
+        let file_scope = index.try_expression_scope_id(&model.expr_ref_in_ast(*self))?;
         let scope = file_scope.to_scope_id(model.db, model.file);
 
-        infer_scope_types(model.db, scope).expression_type(*self)
+        infer_complete_scope_types(model.db, scope).try_expression_type(*self)
     }
 }
 
@@ -431,7 +535,7 @@ macro_rules! impl_expression_has_type {
     ($ty: ty) => {
         impl HasType for $ty {
             #[inline]
-            fn inferred_type<'db>(&self, model: &SemanticModel<'db>) -> Type<'db> {
+            fn inferred_type<'db>(&self, model: &SemanticModel<'db>) -> Option<Type<'db>> {
                 let expression_ref = ExprRef::from(self);
                 expression_ref.inferred_type(model)
             }
@@ -474,7 +578,7 @@ impl_expression_has_type!(ast::ExprSlice);
 impl_expression_has_type!(ast::ExprIpyEscapeCommand);
 
 impl HasType for ast::Expr {
-    fn inferred_type<'db>(&self, model: &SemanticModel<'db>) -> Type<'db> {
+    fn inferred_type<'db>(&self, model: &SemanticModel<'db>) -> Option<Type<'db>> {
         match self {
             Expr::BoolOp(inner) => inner.inferred_type(model),
             Expr::Named(inner) => inner.inferred_type(model),
@@ -525,9 +629,9 @@ macro_rules! impl_binding_has_ty_def {
 
         impl HasType for $ty {
             #[inline]
-            fn inferred_type<'db>(&self, model: &SemanticModel<'db>) -> Type<'db> {
+            fn inferred_type<'db>(&self, model: &SemanticModel<'db>) -> Option<Type<'db>> {
                 let binding = HasDefinition::definition(self, model);
-                binding_type(model.db, binding)
+                Some(binding_type(model.db, binding))
             }
         }
     };
@@ -538,14 +642,15 @@ impl_binding_has_ty_def!(ast::StmtClassDef);
 impl_binding_has_ty_def!(ast::Parameter);
 impl_binding_has_ty_def!(ast::ParameterWithDefault);
 impl_binding_has_ty_def!(ast::ExceptHandlerExceptHandler);
+impl_binding_has_ty_def!(ast::TypeParamTypeVar);
 
 impl HasType for ast::Alias {
-    fn inferred_type<'db>(&self, model: &SemanticModel<'db>) -> Type<'db> {
+    fn inferred_type<'db>(&self, model: &SemanticModel<'db>) -> Option<Type<'db>> {
         if &self.name == "*" {
-            return Type::Never;
+            return Some(Type::Never);
         }
         let index = semantic_index(model.db, model.file);
-        binding_type(model.db, index.expect_single_definition(self))
+        Some(binding_type(model.db, index.expect_single_definition(self)))
     }
 }
 
@@ -557,8 +662,10 @@ impl HasTrackedScope for ast::Expr {}
 impl HasTrackedScope for ast::ExprRef<'_> {}
 impl HasTrackedScope for &ast::ExprRef<'_> {}
 
-// See https://github.com/astral-sh/ty/issues/572 why this implementation exists
-// even when we never register identifiers during semantic index building.
+// We never explicitly register the scope of an `Identifier`.
+// However, `ExpressionsScopeMap` stores the text ranges of each scope.
+// That allows us to look up the identifier's scope for as long as it's
+// inside an expression (because the ranges overlap).
 impl HasTrackedScope for ast::Identifier {}
 
 #[cfg(test)]
@@ -581,7 +688,7 @@ mod tests {
 
         let function = ast.suite()[0].as_function_def_stmt().unwrap();
         let model = SemanticModel::new(&db, foo);
-        let ty = function.inferred_type(&model);
+        let ty = function.inferred_type(&model).unwrap();
 
         assert!(ty.is_function_literal());
 
@@ -600,7 +707,7 @@ mod tests {
 
         let class = ast.suite()[0].as_class_def_stmt().unwrap();
         let model = SemanticModel::new(&db, foo);
-        let ty = class.inferred_type(&model);
+        let ty = class.inferred_type(&model).unwrap();
 
         assert!(ty.is_class_literal());
 
@@ -621,7 +728,7 @@ mod tests {
         let import = ast.suite()[0].as_import_from_stmt().unwrap();
         let alias = &import.names[0];
         let model = SemanticModel::new(&db, bar);
-        let ty = alias.inferred_type(&model);
+        let ty = alias.inferred_type(&model).unwrap();
 
         assert!(ty.is_class_literal());
 
